#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2020/11/5 14:55
Desc: 历年世界 500 强榜单数据
http://www.fortunechina.com/fortune500/index.htm
特殊情况说明：
2010年由于网页端没有公布公司所属的国家, 故 2010 年数据没有国家这列
"""
import json

import pandas as pd
import requests
from bs4 import BeautifulSoup

from mssdk.fortune.cons import *


def fortune_rank(year: str = "2015") -> pd.DataFrame:
    """
    财富 500 强公司从 1996 年开始的排行榜
    http://www.fortunechina.com/fortune500/index.htm
    :param year: str 年份
    :return: pandas.DataFrame
    """
    if int(year) in [item for item in range(2014, 2021)] + [item for item in range(1996, 2007)]:
        if year in ["2006", "2007"]:
            res = requests.get(eval("url_" + year))
            res.encoding = "utf-8"
            df = pd.read_html(res.text)[0].iloc[1:, 2:]
            df.columns = pd.read_html(res.text)[0].iloc[0, 2:].tolist()
            return df
        elif year in ["1996", "1997", "1998", "1999", "2000", "2001", "2003", "2004", "2005"]:
            res = requests.get(eval("url_" + year))
            res.encoding = "utf-8"
            df = pd.read_html(res.text)[0].iloc[1:-1, 1:]
            df.columns = pd.read_html(res.text)[0].iloc[0, 1:].tolist()
            return df
        elif year in ["2002"]:
            res = requests.get(eval("url_" + year))
            res.encoding = "utf-8"
            df = pd.read_html(res.text)[0].iloc[1:, 1:]
            df.columns = pd.read_html(res.text)[0].iloc[0, 1:].tolist()
            return df
        elif year in ["2020"]:
            res = requests.get(eval("url_" + year))
            res.encoding = "utf-8"
            df = pd.read_html(res.text)[0].iloc[:, :-1]
            return df
        else:
            res = requests.get(eval("url_" + year))
            res.encoding = "utf-8"
            df = pd.read_html(res.text)[0].iloc[:, 2:]
            return df
    elif int(year) in [item for item in range(2010, 2014)]:
        if int(year) == 2011:
            res = requests.get(eval(f"url_{2011}"))
            res.encoding = "utf-8"
            df = pd.read_html(res.text)[0].iloc[:, 2:]
            temp_df = df
            for page in range(2, 6):
                # page = 1
                res = requests.get(eval(f"url_{2011}").rsplit(".", maxsplit=1)[0] + "_" + str(page) + ".htm")
                res.encoding = "utf-8"
                df = pd.read_html(res.text)[0].iloc[:, 2:]
                temp_df = temp_df.append(df, ignore_index=True)
            temp_df.columns = ["公司名称", "营业收入百万美元", "利润百万美元", "国家地区"]
            return temp_df
        res = requests.get(eval(f"url_{year}"))
        res.encoding = "utf-8"
        df = pd.read_html(res.text)[0].iloc[:, 2:]
        temp_df = df
        for page in range(2, 6):
            # page = 1
            res = requests.get(eval(f"url_{year}").rsplit(".", maxsplit=1)[0] + "_" + str(page) + ".htm")
            res.encoding = "utf-8"
            df = pd.read_html(res.text)[0].iloc[:, 2:]
            temp_df = temp_df.append(df, ignore_index=True)
        df = temp_df
        return df
    elif int(year) in [item for item in range(2008, 2010)]:
        res = requests.get(eval(f"url_{year}"))
        res.encoding = "utf-8"
        df = pd.read_html(res.text)[0].iloc[1:, 2:]
        df.columns = pd.read_html(res.text)[0].iloc[0, 2:].tolist()
        temp_df = df
        for page in range(2, 11):
            # page = 1
            res = requests.get(eval(f"url_{year}").rsplit(".", maxsplit=1)[0] + "_" + str(page) + ".htm")
            res.encoding = "utf-8"
            text_df = pd.read_html(res.text)[0]
            df = text_df.iloc[1:, 2:]
            df.columns = text_df.iloc[0, 2:]
            temp_df = temp_df.append(df, ignore_index=True)
        df = temp_df
        return df
    elif int(year) == 2007:
        res = requests.get(eval(f"url_{year}"))
        res.encoding = "utf-8"
        df = pd.read_html(res.text)[0].iloc[1:, 1:]
        df.columns = pd.read_html(res.text)[0].iloc[0, 1:].tolist()
        temp_df = df
        for page in range(2, 11):
            # page = 1
            res = requests.get(eval(f"url_{year}").rsplit(".", maxsplit=1)[0] + "_" + str(page) + ".htm")
            res.encoding = "utf-8"
            text_df = pd.read_html(res.text)[0]
            df = text_df.iloc[1:, 1:]
            df.columns = text_df.iloc[0, 1:]
            temp_df = temp_df.append(df, ignore_index=True)
        df = temp_df
        return df


def fortune_rank_eng(year: str = "1995") -> pd.DataFrame:
    """
    注意你的网速
    https://fortune.com/global500/
    https://fortune.com/global500/2012/search/
    :param year: "1995"
    :type year: str
    :return: 历年排名
    :rtype: pandas.DataFrame
    """
    url = f"https://fortune.com/global500/{year}/search/"
    res = requests.get(url)
    soup = BeautifulSoup(res.text, "lxml")
    code = json.loads(soup.find("script", attrs={"type": "application/ld+json"}).string)["identifier"]
    url = f"https://content.fortune.com/wp-json/irving/v1/data/franchise-search-results"
    params = {
        "list_id": code,
        "token": "Zm9ydHVuZTpCcHNyZmtNZCN5SndjWkkhNHFqMndEOTM=",
    }
    res = requests.get(url, params=params)
    big_df = pd.DataFrame()
    for i in range(len(res.json()[1]["items"][0]['fields'])):
        temp_df = pd.DataFrame([item["fields"][i] for item in res.json()[1]["items"]])
        big_df[temp_df["key"].values[0]] = temp_df["value"]
    big_df["rank"] = big_df["rank"].astype(int)
    big_df.sort_values("rank", inplace=True)
    big_df.reset_index(drop=True, inplace=True)
    return big_df


if __name__ == '__main__':
    fortune_rank_df = fortune_rank(year="2020")  # 2010 不一样
    print(fortune_rank_df)
    for i_year in range(1995, 2020):
        print(i_year)
        fortune_eng_df = fortune_rank_eng(year=i_year)
        print(fortune_eng_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/7/16 12:02
Desc: 东方财富网-数据中心-新股数据-打新收益率
东方财富网-数据中心-新股数据-打新收益率
http://data.eastmoney.com/xg/xg/dxsyl.html
东方财富网-数据中心-新股数据-新股申购与中签查询
http://data.eastmoney.com/xg/xg/default_2.html
"""
from mssdk.utils import demjson
import pandas as pd
import requests
from tqdm import tqdm


def _get_page_num_dxsyl(market: str = "上海主板") -> int:
    """
    东方财富网-数据中心-新股数据-打新收益率-总页数
    http://data.eastmoney.com/xg/xg/dxsyl.html
    :param market: choice of {"上海主板", "创业板", "深圳主板"}
    :type market: str
    :return: 总页数
    :rtype: int
    """
    market_map = {"上海主板": "2", "创业板": "3", "深圳主板": "4"}
    url = "http://datainterface.eastmoney.com/EM_DataCenter/JS.aspx"
    params = {
        "type": "NS",
        "sty": "NSDXSYL",
        "st": "16",
        "sr": "-1",
        "p": "1",
        "ps": "50",
        "js": "var oyfyNYmO={pages:(pc),data:[(x)]}",
        "stat": market_map[market],
        "rt": "52898446",
    }
    res = requests.get(url, params=params)
    data_json = demjson.decode(res.text[res.text.find("={") + 1 :])
    return data_json["pages"]


def stock_em_dxsyl(market: str = "上海主板") -> pd.DataFrame:
    """
    东方财富网-数据中心-新股数据-打新收益率
    http://data.eastmoney.com/xg/xg/dxsyl.html
    :param market: choice of {"上海主板", "创业板", "深圳主板"}
    :type market: str
    :return: 指定市场的打新收益率数据
    :rtype: pandas.DataFrame
    """
    market_map = {"上海主板": "2", "创业板": "3", "深圳主板": "4"}
    url = "http://datainterface.eastmoney.com/EM_DataCenter/JS.aspx"
    page_num = _get_page_num_dxsyl(market=market)
    temp_df = pd.DataFrame()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params = {
            "type": "NS",
            "sty": "NSDXSYL",
            "st": "16",
            "sr": "-1",
            "p": str(page),
            "ps": "50",
            "js": "({data:[(x)],pages:(pc)})",
            "stat": market_map[market],
            "rt": "52898446",
        }
        res = requests.get(url, params=params)
        data_text = res.text
        data_json = demjson.decode(data_text[data_text.find("(") + 1 :-1])
        temp_df = temp_df.append(pd.DataFrame(data_json["data"]), ignore_index=True)
    temp_df = temp_df.iloc[:, 0].str.split(",", expand=True)
    temp_df.columns = [
        "股票代码",
        "股票简称",
        "发行价",
        "最新价",
        "网上发行中签率",
        "网上有效申购股数",
        "网上有效申购户数",
        "网上超额认购倍数",
        "网下配售中签率",
        "网下有效申购股数",
        "网下有效申购户数",
        "网下配售认购倍数",
        "总发行数量",
        "开盘溢价",
        "首日涨幅",
        "打新收益",
        "上市日期",
        "市场",
    ]
    temp_df["发行价"] = pd.to_numeric(temp_df["发行价"])
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"])
    temp_df["网上发行中签率"] = pd.to_numeric(temp_df["网上发行中签率"])
    temp_df["网上有效申购股数"] = pd.to_numeric(temp_df["网上有效申购股数"])
    temp_df["网上有效申购户数"] = pd.to_numeric(temp_df["网上有效申购户数"])
    temp_df["网上超额认购倍数"] = pd.to_numeric(temp_df["网上超额认购倍数"])
    temp_df["网下配售中签率"] = pd.to_numeric(temp_df["网下配售中签率"])
    temp_df["网下有效申购股数"] = pd.to_numeric(temp_df["网下有效申购股数"])
    temp_df["网下有效申购户数"] = pd.to_numeric(temp_df["网下有效申购户数"])
    temp_df["网下配售认购倍数"] = pd.to_numeric(temp_df["网下配售认购倍数"])
    temp_df["总发行数量"] = pd.to_numeric(temp_df["总发行数量"])
    temp_df["开盘溢价"] = pd.to_numeric(temp_df["开盘溢价"])
    temp_df["首日涨幅"] = pd.to_numeric(temp_df["首日涨幅"])
    temp_df["打新收益"] = pd.to_numeric(temp_df["打新收益"])
    return temp_df


def stock_em_xgsglb(market: str = "沪市A股") -> pd.DataFrame:
    """
    新股申购与中签查询
    http://data.eastmoney.com/xg/xg/default_2.html
    :param market: choice of {"全部股票", "沪市A股", "科创板", "深市A股", "创业板"}
    :type market: str
    :return: 新股申购与中签数据
    :rtype: pandas.DataFrame
    """
    market_map = {
        "全部股票": """(APPLY_DATE>'2010-01-01')""",
        "沪市A股": """(APPLY_DATE>'2010-01-01')(SECURITY_TYPE_CODE in ("058001001","058001008"))(TRADE_MARKET_CODE in ("069001001001","069001001003","069001001006"))""",
        "科创板": """(APPLY_DATE>'2010-01-01')(SECURITY_TYPE_CODE in ("058001001","058001008"))(TRADE_MARKET_CODE="069001001006")""",
        "深市A股": """(APPLY_DATE>'2010-01-01')(SECURITY_TYPE_CODE="058001001")(TRADE_MARKET_CODE in ("069001002001","069001002002","069001002003","069001002005"))""",
        "创业板": """(APPLY_DATE>'2010-01-01')(SECURITY_TYPE_CODE="058001001")(TRADE_MARKET_CODE="069001002002")""",
    }
    url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        'sortColumns': 'APPLY_DATE,SECURITY_CODE',
        'sortTypes': '-1,-1',
        'pageSize': '5000',
        'pageNumber': '1',
        'reportName': 'RPTA_APP_IPOAPPLY',
        'columns': 'SECURITY_CODE,SECURITY_NAME,TRADE_MARKET_CODE,APPLY_CODE,TRADE_MARKET,MARKET_TYPE,ORG_TYPE,ISSUE_NUM,ONLINE_ISSUE_NUM,OFFLINE_PLACING_NUM,TOP_APPLY_MARKETCAP,PREDICT_ONFUND_UPPER,ONLINE_APPLY_UPPER,PREDICT_ONAPPLY_UPPER,ISSUE_PRICE,LATELY_PRICE,CLOSE_PRICE,APPLY_DATE,BALLOT_NUM_DATE,BALLOT_PAY_DATE,LISTING_DATE,AFTER_ISSUE_PE,ONLINE_ISSUE_LWR,INITIAL_MULTIPLE,INDUSTRY_PE_NEW,OFFLINE_EP_OBJECT,CONTINUOUS_1WORD_NUM,TOTAL_CHANGE,PROFIT,LIMIT_UP_PRICE,INFO_CODE,OPEN_PRICE,LD_OPEN_PREMIUM,LD_CLOSE_CHANGE,TURNOVERRATE,LD_HIGH_CHANG,LD_AVERAGE_PRICE,OPEN_DATE,OPEN_AVERAGE_PRICE,PREDICT_PE,PREDICT_ISSUE_PRICE2,PREDICT_ISSUE_PRICE,PREDICT_ISSUE_PRICE1,PREDICT_ISSUE_PE,PREDICT_PE_THREE,ONLINE_APPLY_PRICE,MAIN_BUSINESS',
        'filter': market_map[market],
        'source': 'WEB',
        'client': 'WEB',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json['result']['pages']
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_page+1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json['result']['data'])
        big_df = big_df.append(temp_df, ignore_index=True)
    big_df.columns = [
        "股票代码",
        "股票简称",
        "_",
        "申购代码",
        "_",
        "_",
        "_",
        "发行总数",
        "网上发行",
        "_",
        "顶格申购需配市值",
        "_",
        "申购上限",
        "_",
        "发行价格",
        "最新价",
        "首日收盘价",
        "申购日期",
        "中签号公布日",
        "中签缴款日期",
        "上市日期",
        "发行市盈率",
        "中签率",
        "询价累计报价倍数",
        "_",
        "配售对象报价家数",
        "连续一字板数量",
        "涨幅",
        "每中一签获利",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "行业市盈率",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "股票代码",
            "股票简称",
            "申购代码",
            "发行总数",
            "网上发行",
            "顶格申购需配市值",
            "申购上限",
            "发行价格",
            "最新价",
            "首日收盘价",
            "申购日期",
            "中签号公布日",
            "中签缴款日期",
            "上市日期",
            "发行市盈率",
            "行业市盈率",
            "中签率",
            "询价累计报价倍数",
            "配售对象报价家数",
            "连续一字板数量",
            "涨幅",
            "每中一签获利",
        ]
    ]
    big_df['申购日期'] = pd.to_datetime(big_df['申购日期']).dt.date
    big_df['中签号公布日'] = pd.to_datetime(big_df['中签号公布日']).dt.date
    big_df['中签缴款日期'] = pd.to_datetime(big_df['中签缴款日期']).dt.date
    big_df['发行总数'] = pd.to_numeric(big_df['发行总数'])
    big_df['网上发行'] = pd.to_numeric(big_df['网上发行'])
    big_df['顶格申购需配市值'] = pd.to_numeric(big_df['顶格申购需配市值'])
    big_df['申购上限'] = pd.to_numeric(big_df['申购上限'])
    big_df['发行价格'] = pd.to_numeric(big_df['发行价格'])
    big_df['最新价'] = pd.to_numeric(big_df['最新价'])
    big_df['首日收盘价'] = pd.to_numeric(big_df['首日收盘价'])
    big_df['发行市盈率'] = pd.to_numeric(big_df['发行市盈率'])
    big_df['行业市盈率'] = pd.to_numeric(big_df['行业市盈率'])
    big_df['中签率'] = pd.to_numeric(big_df['中签率'])
    big_df['询价累计报价倍数'] = pd.to_numeric(big_df['询价累计报价倍数'])
    big_df['配售对象报价家数'] = pd.to_numeric(big_df['配售对象报价家数'])
    big_df['涨幅'] = pd.to_numeric(big_df['涨幅'])
    big_df['每中一签获利'] = pd.to_numeric(big_df['每中一签获利'])
    return big_df


if __name__ == "__main__":
    stock_em_dxsyl_df = stock_em_dxsyl(market="上海主板")
    print(stock_em_dxsyl_df)

    stock_em_dxsyl_df = stock_em_dxsyl(market="创业板")
    print(stock_em_dxsyl_df)

    # stock_em_dxsyl_df = stock_em_dxsyl(market="深圳主板")
    # print(stock_em_dxsyl_df)

    stock_em_xgsglb_df = stock_em_xgsglb(market="全部股票")
    print(stock_em_xgsglb_df)

    stock_em_xgsglb_df = stock_em_xgsglb(market="沪市A股")
    print(stock_em_xgsglb_df)

    stock_em_xgsglb_df = stock_em_xgsglb(market="科创板")
    print(stock_em_xgsglb_df)

    stock_em_xgsglb_df = stock_em_xgsglb(market="深市A股")
    print(stock_em_xgsglb_df)

    stock_em_xgsglb_df = stock_em_xgsglb(market="创业板")
    print(stock_em_xgsglb_df)



# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Date: 2020/01/02 17:37
Desc: 获取交易法门-工具: https://www.jiaoyifamen.com/tools/
交易法门首页: https://www.jiaoyifamen.com/

# 交易法门-工具-套利分析
交易法门-工具-套利分析-跨期价差(自由价差)
交易法门-工具-套利分析-自由价比
交易法门-工具-套利分析-多腿组合
交易法门-工具-套利分析-FullCarry
交易法门-工具-套利分析-套利价差矩阵

# 交易法门-工具-资讯汇总
交易法门-工具-资讯汇总-研报查询
交易法门-工具-资讯汇总-交易日历

# 交易法门-工具-持仓分析
交易法门-工具-持仓分析-期货持仓
交易法门-工具-持仓分析-席位持仓
交易法门-工具-持仓分析-持仓季节性

# 交易法门-工具-资金分析
交易法门-工具-资金分析-资金流向
交易法门-工具-资金分析-沉淀资金
交易法门-工具-资金分析-资金季节性
交易法门-工具-资金分析-成交排名

# 交易法门-工具-席位分析
交易法门-工具-席位分析-持仓结构
交易法门-工具-席位分析-持仓成本
交易法门-工具-席位分析-建仓过程

# 交易法门-工具-仓单分析
交易法门-工具-仓单分析-仓单日报
交易法门-工具-仓单分析-仓单查询
交易法门-工具-仓单分析-虚实盘比日报
交易法门-工具-仓单分析-虚实盘比查询

# 交易法门-工具-期限分析
交易法门-工具-期限分析-基差日报
交易法门-工具-期限分析-基差分析
交易法门-工具-期限分析-期限结构
交易法门-工具-期限分析-价格季节性

# 交易法门-工具-行情分析
交易法门-工具-行情分析-行情数据

# 交易法门-工具-交易规则
交易法门-工具-交易规则-限仓规定
交易法门-工具-交易规则-仓单有效期
交易法门-工具-交易规则-品种手册
"""
import time

import matplotlib.pyplot as plt
import pandas as pd
import requests

from mssdk.futures_derivative.cons import (
    csa_payload,
    csa_url_spread,
    csa_url_ratio,
    csa_url_customize,
)
from mssdk.futures_derivative.jyfm_login_func import jyfm_login


# pd.set_option('display.max_columns', None)


# 交易法门-工具-套利分析
def jyfm_tools_futures_spread(
    type_1="RB", type_2="RB", code_1="01", code_2="05", headers="", plot=True
):
    """
    交易法门-工具-套利分析-跨期价差(自由价差)
    :param type_1: str
    :param type_2: str
    :param code_1: str
    :param code_2: str
    :param plot: Bool
    :return: pandas.Series or pic
    """
    csa_payload_his = csa_payload.copy()
    csa_payload_his.update({"type1": type_1})
    csa_payload_his.update({"type2": type_2})
    csa_payload_his.update({"code1": code_1})
    csa_payload_his.update({"code2": code_2})
    res = requests.get(csa_url_spread, params=csa_payload_his, headers=headers)
    data_json = res.json()
    data_df = pd.DataFrame([data_json["category"], data_json["value"]]).T
    data_df.index = pd.to_datetime(data_df.iloc[:, 0])
    data_df = data_df.iloc[:, 1]
    data_df.name = "value"
    if plot:
        data_df.plot()
        plt.legend(loc="best")
        plt.xlabel("date")
        plt.ylabel("value")
        plt.show()
        return data_df
    else:
        return data_df


def jyfm_tools_futures_ratio(
    type_1="RB", type_2="RB", code_1="01", code_2="05", headers="", plot=True
):
    """
    交易法门-工具-套利分析-自由价比
    :param type_1: str
    :param type_2: str
    :param code_1: str
    :param code_2: str
    :param plot: Bool
    :return: pandas.Series or pic
    2013-01-04   -121
    2013-01-07   -124
    2013-01-08   -150
    2013-01-09   -143
    2013-01-10   -195
                 ...
    2019-10-21    116
    2019-10-22    126
    2019-10-23    123
    2019-10-24    126
    2019-10-25    134
    """
    csa_payload_his = csa_payload.copy()
    csa_payload_his.update({"type1": type_1})
    csa_payload_his.update({"type2": type_2})
    csa_payload_his.update({"code1": code_1})
    csa_payload_his.update({"code2": code_2})
    res = requests.get(csa_url_ratio, params=csa_payload_his, headers=headers)
    data_json = res.json()
    data_df = pd.DataFrame([data_json["category"], data_json["value"]]).T
    data_df.index = pd.to_datetime(data_df.iloc[:, 0])
    data_df = data_df.iloc[:, 1]
    data_df.name = "value"
    if plot:
        data_df.plot()
        plt.legend(loc="best")
        plt.xlabel("date")
        plt.ylabel("value")
        plt.show()
        return data_df
    else:
        return data_df


def jyfm_tools_futures_customize(
    formula="RB01-1.6*I01-0.5*J01-1200", headers="", plot=True
):
    """
    交易法门-工具-套利分析-多腿组合
    :param formula: str
    :param plot: Bool
    :return: pandas.Series or pic
    """
    params = {"formula": formula}
    res = requests.get(csa_url_customize, params=params, headers=headers)
    data_json = res.json()
    data_df = pd.DataFrame([data_json["category"], data_json["value"]]).T
    data_df.index = pd.to_datetime(data_df.iloc[:, 0])
    data_df = data_df.iloc[:, 1]
    data_df.name = "value"
    if plot:
        data_df.plot()
        plt.legend(loc="best")
        plt.xlabel("date")
        plt.ylabel("value")
        plt.show()
        return data_df
    else:
        return data_df


def jyfm_tools_futures_full_carry(
    begin_code="05", end_code="09", ratio="4", headers=""
):
    """
    交易法门-工具-套利分析-FullCarry
    https://www.jiaoyifamen.com/tools/future/full/carry?beginCode=05&endCode=09&ratio=4
    注: 正向转抛成本主要是仓储费和资金成本，手续费占比很小，故忽略。增值税不确定，故也未列入计算。使用该表时注意仓单有效期问题、升贴水问题以及生鲜品种其他较高费用的问题。实际Full Carry水平要略高于这里的测算水平。
    :param begin_code: 开始月份
    :type begin_code: str
    :param end_code: 结束月份
    :type end_code: str
    :param ratio: 百分比, 这里输入绝对值
    :type ratio: str
    :param headers: 请求头
    :type headers: dict
    :return: 正向市场转抛成本估算
    :rtype: pandas.DataFrame
    """
    url = "https://www.jiaoyifamen.com/tools/future/full/carry"
    params = {
        "beginCode": begin_code,
        "endCode": end_code,
        "ratio": ratio,
    }
    res = requests.get(url, params=params, headers=headers)
    return pd.DataFrame(res.json()["table_data"])


def jyfm_tools_futures_arbitrage_matrix(
    category="1", type1="RB", type2="RB", headers=""
):
    """
    交易法门-工具-套利分析-跨期价差矩阵
    https://www.jiaoyifamen.com/tools/future/arbitrage/matrix
    :param category: 1: 跨期价差; 2: 自由价差; 3: 自由价比
    :type category: str
    :param type1: 种类一
    :type type1: str
    :param type2: 种类二
    :type type2: str
    :param headers: 请求头
    :type headers: dict
    :return: 对应的矩阵
    :rtype: pandas.DataFrame
    """
    url = "https://www.jiaoyifamen.com/tools/future/arbitrage/matrix"
    params = {
        "category": category,
        "type1": type1,
        "type2": type2,
        "_": "1583846468579",
    }
    res = requests.get(url, params=params, headers=headers)
    return pd.DataFrame(res.json()["data"])


def jyfm_exchange_symbol_dict():
    jyfm_exchange_symbol_dict_inner = {
        "中国金融期货交易所": {
            "TF": "五债",
            "T": "十债",
            "IC": "中证500",
            "IF": "沪深300",
            "IH": "上证50",
            "TS": "二债",
        },
        "郑州商品交易所": {
            "FG": "玻璃",
            "RS": "菜籽",
            "CF": "棉花",
            "LR": "晚稻",
            "CJ": "红枣",
            "JR": "粳稻",
            "ZC": "动力煤",
            "TA": "PTA",
            "SA": "纯碱",
            "AP": "苹果",
            "WH": "强麦",
            "SF": "硅铁",
            "MA": "甲醇",
            "CY": "棉纱",
            "RI": "早稻",
            "OI": "菜油",
            "SM": "硅锰",
            "RM": "菜粕",
            "UR": "尿素",
            "PM": "普麦",
            "SR": "白糖",
        },
        "大连商品交易所": {
            "PP": "PP",
            "RR": "粳米",
            "BB": "纤板",
            "A": "豆一",
            "EG": "乙二醇",
            "B": "豆二",
            "C": "玉米",
            "JM": "焦煤",
            "I": "铁矿",
            "J": "焦炭",
            "L": "塑料",
            "M": "豆粕",
            "P": "棕榈",
            "CS": "淀粉",
            "V": "PVC",
            "Y": "豆油",
            "JD": "鸡蛋",
            "FB": "胶板",
            "EB": "苯乙烯",
        },
        "上海期货交易所": {
            "SS": "不锈钢",
            "RU": "橡胶",
            "AG": "沪银",
            "AL": "沪铝",
            "FU": "燃油",
            "RB": "螺纹",
            "CU": "沪铜",
            "PB": "沪铅",
            "BU": "沥青",
            "AU": "沪金",
            "ZN": "沪锌",
            "SN": "沪锡",
            "HC": "热卷",
            "NI": "沪镍",
            "WR": "线材",
            "SP": "纸浆",
        },
        "上海国际能源交易中心": {"SC": "原油", "NR": "20号胶"},
    }
    return jyfm_exchange_symbol_dict_inner


# 交易法门-工具-资讯汇总
def jyfm_tools_research_query(limit="100", headers=""):
    """
    交易法门-工具-资讯汇总-研报查询
    https://www.jiaoyifamen.com/tools/research/qryPageList
    :param limit: 返回条数
    :type limit: str
    :return: 返回研报信息数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.jiaoyifamen.com/tools/research/qryPageList"
    params = {
        "page": "1",
        "limit": limit,
    }
    res = requests.get(url, params=params, headers=headers)
    return pd.DataFrame(res.json()["data"])


def jyfm_tools_trade_calendar(trade_date="2020-01-03", headers=""):
    """
    交易法门-工具-资讯汇总-交易日历
    此函数可以返回未来的交易日历数据
    https://www.jiaoyifamen.com/tools/trade-calendar/events
    :param trade_date: 指定交易日
    :type trade_date: str
    :return: 返回指定交易日的交易日历数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.jiaoyifamen.com/tools/trade-calendar/events"
    params = {
        "page": "1",
        "limit": "1000",
        "day": trade_date,
    }
    res = requests.get(url, params=params, headers=headers)
    return pd.DataFrame(res.json()["data"])


# 交易法门-工具-持仓分析
def jyfm_tools_position_detail(
    symbol="JM", code="jm2005", trade_date="2020-01-03", headers=""
):
    """
    交易法门-工具-持仓分析-期货持仓
    :param symbol: 指定品种
    :type symbol: str
    :param code: 指定合约
    :type code: str
    :param trade_date: 指定交易日
    :type trade_date: str
    :param headers: headers with cookies
    :type headers:dict
    :return: 指定品种的指定合约的指定交易日的期货持仓数据
    :rtype: pandas.DataFrame
    """
    url = f"https://www.jiaoyifamen.com/tools/position/details/{symbol}?code={code}&day={trade_date}&_=1578040551329"
    res = requests.get(url, headers=headers)
    return pd.DataFrame(res.json()["short_rank_table"])


def jyfm_tools_position_seat(seat="永安期货", trade_date="2020-01-03", headers=""):
    """
    交易法门-工具-持仓分析-持仓分析-席位持仓
    :param seat: 指定期货公司
    :type seat: str
    :param trade_date: 具体交易日
    :type trade_date: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定期货公司指定交易日的席位持仓数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.jiaoyifamen.com/tools/position/seat"
    params = {
        "seat": seat,
        "day": trade_date,
        "type": "",
        "_": "1578040989932",
    }
    res = requests.get(url, params=params, headers=headers)
    return pd.DataFrame(res.json()["data"])


def jyfm_tools_position_season(symbol="RB", code="05", headers=""):
    """
    交易法门-工具-持仓分析-持仓分析-持仓季节性
    https://www.jiaoyifamen.com/tools/position/season
    :param symbol: 具体品种
    :type symbol: str
    :param code: 具体合约月份
    :type code: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 合约持仓季节性规律
    :rtype: pandas.DataFrame
    """
    url = "https://www.jiaoyifamen.com/tools/position/season"
    params = {
        "type": symbol,
        "code": code,
    }
    res = requests.get(url, params=params, headers=headers)
    data_json = res.json()
    temp_df = pd.DataFrame(
        [
            data_json["year2013"],
            data_json["year2014"],
            data_json["year2015"],
            data_json["year2016"],
            data_json["year2017"],
            data_json["year2018"],
            data_json["year2019"],
            data_json["year2020"],
        ],
        columns=data_json["dataCategory"],
    ).T
    temp_df.columns = ["2013", "2014", "2015", "2016", "2017", "2018", "2019", "2020"]
    return temp_df


# 交易法门-工具-资金分析
def jyfm_tools_position_fund_direction(
    trade_date="2020-02-24", indicator="期货品种资金流向排名", headers=""
):
    """
    交易法门-工具-资金分析-资金流向
    https://www.jiaoyifamen.com/tools/position/fund/?day=2020-01-08
    :param trade_date: 指定交易日
    :type trade_date: str
    :param indicator: "期货品种资金流向排名" or "期货主力合约资金流向排名"
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定交易日的资金流向数据
    :rtype: pandas.DataFrame
    """
    params = {
        "day": trade_date,
    }
    url = "https://www.jiaoyifamen.com/tools/position/fund/"
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    if indicator == "期货品种资金流向排名":
        return pd.DataFrame(
            [
                [data_json["tradingDay"]] * len(data_json["flowCategory"]),
                data_json["flowCategory"],
                data_json["flowValue"],
            ],
            index=["date", "symbol", "fund"],
        ).T
    else:
        return pd.DataFrame(
            [
                [data_json["tradingDay"]] * len(data_json["dominantFlowCategory"]),
                data_json["dominantFlowCategory"],
                data_json["dominantFlowValue"],
            ],
            index=["date", "symbol", "fund"],
        ).T


def jyfm_tools_position_fund_down(
    trade_date="2020-02-24", indicator="期货品种沉淀资金排名", headers=""
):
    """
    交易法门-工具-资金分析-沉淀资金
    https://www.jiaoyifamen.com/tools/position/fund/?day=2020-01-08
    :param trade_date: 指定交易日
    :type trade_date: str
    :param indicator: "期货品种沉淀资金排名" or "期货主力合约沉淀资金排名"
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定交易日的沉淀资金
    :rtype: pandas.DataFrame
    """
    params = {
        "day": trade_date,
    }
    url = "https://www.jiaoyifamen.com/tools/position/fund/"
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    if indicator == "期货品种沉淀资金排名":
        return pd.DataFrame(
            [
                [data_json["tradingDay"]] * len(data_json["precipitationCategory"]),
                data_json["precipitationCategory"],
                data_json["precipitationValue"],
            ],
            index=["date", "symbol", "fund"],
        ).T
    else:
        return pd.DataFrame(
            [
                [data_json["tradingDay"]]
                * len(data_json["dominantPrecipitationCategory"]),
                data_json["dominantPrecipitationCategory"],
                data_json["dominantPrecipitationValue"],
            ],
            index=["date", "symbol", "fund"],
        ).T


def jyfm_tools_position_fund_season(symbol="RB", code="05", headers=""):
    """
    交易法门-工具-资金分析-资金季节性
    https://www.jiaoyifamen.com/tools/position/fund/?day=2020-01-08
    :param symbol: 指定品种
    :type symbol: str
    :param code: 合约到期月
    :type code: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定交易日的资金资金季节性
    :rtype: pandas.DataFrame
    """
    params = {
        "type": symbol,
        "code": code,
    }
    url = "https://www.jiaoyifamen.com/tools/position/fund/season"
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    data_df = pd.DataFrame(
        [
            data_json["dataCategory"],
            data_json["year2013"],
            data_json["year2014"],
            data_json["year2015"],
            data_json["year2016"],
            data_json["year2017"],
            data_json["year2018"],
            data_json["year2019"],
            data_json["year2020"],
        ],
        index=["date", "2013", "2014", "2015", "2016", "2017", "2018", "2019", "2020"],
    ).T
    return data_df


def jyfm_tools_position_fund_deal(
    trade_date="2020-02-24", indicator="期货品种成交量排名", headers=""
):
    """
    交易法门-工具-资金分析-成交排名
    https://www.jiaoyifamen.com/tools/position/fund/?day=2020-01-08
    :param trade_date: 指定交易日
    :type trade_date: str
    :param indicator: "期货品种成交量排名" or "期货主力合约成交量排名"
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定交易日的资金成交排名
    :rtype: pandas.DataFrame
    """
    params = {
        "day": trade_date,
    }
    url = "https://www.jiaoyifamen.com/tools/position/fund/"
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    if indicator == "期货品种成交量排名":
        return pd.DataFrame(
            [
                [data_json["tradingDay"]] * len(data_json["turnOverCategory"]),
                data_json["turnOverCategory"],
                data_json["turnOverValue"],
            ],
            index=["date", "symbol", "fund"],
        ).T
    else:
        return pd.DataFrame(
            [
                [data_json["tradingDay"]] * len(data_json["dominantTurnOverCategory"]),
                data_json["dominantTurnOverCategory"],
                data_json["dominantTurnOverValue"],
            ],
            index=["date", "symbol", "fund"],
        ).T


# 交易法门-工具-席位分析-持仓结构
def jyfm_tools_position_structure(
    trade_date="2020-03-02", seat="永安期货", indicator="持仓变化", headers=""
):
    """
    交易法门-工具-席位分析-持仓结构
    https://www.jiaoyifamen.com/tools/position/seat
    :param trade_date: 指定交易日
    :type trade_date: str
    :param seat: broker name, e.g., seat="永安期货"
    :type seat: str
    :param indicator: 持仓变化，净持仓分布，总持仓分布; 持仓变化总，净持仓分布总，总持仓分布总
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定交易日指定机构的持仓结构
    :rtype: pandas.DataFrame
    """
    indicator_dict = {"持仓变化": 1, "净持仓分布": 2, "总持仓分布": 3}
    params = {
        "seat": seat,
        "day": trade_date,
        "type": indicator_dict[indicator],
        "_": int(time.time() * 1000),
    }
    url = "https://www.jiaoyifamen.com/tools/position/struct"
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    if indicator == "持仓变化":
        return pd.DataFrame(data_json["varieties"])
    if indicator == "净持仓分布":
        return pd.DataFrame(data_json["varieties"])
    if indicator == "总持仓分布":
        return pd.DataFrame(data_json["varieties"])
    if indicator == "持仓变化总":
        return pd.DataFrame(data_json["ratio"])
    if indicator == "净持仓分布总":
        return pd.DataFrame(data_json["ratio"])
    if indicator == "总持仓分布总":
        return pd.DataFrame(data_json["ratio"])


# 交易法门-工具-席位分析-持仓成本
def jyfm_tools_position_seat_cost(seat="永安期货", symbol="RB", code="10", headers=""):
    """
    交易法门-工具-席位分析-持仓成本
    https://www.jiaoyifamen.com/tools/position/seat
    :param seat: broker name, e.g., seat="永安期货"
    :type seat: str
    :param symbol: e.g., RB
    :type symbol: str
    :param code: e.g., 10
    :type code: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定交易日指定机构的持仓结构
    :rtype: pandas.DataFrame
    """
    params = {
        "seat": seat,
        "type": symbol,
        "code": code,
        "_": int(time.time() * 1000),
    }
    url = "https://www.jiaoyifamen.com/tools/position/seat-cost"
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    return pd.DataFrame(data_json["seatCost"])


# 交易法门-工具-席位分析-建仓过程
def jyfm_tools_position_interest_process(seat="永安期货", symbol="RB", instrument="rb2005", indicator="建仓过程", headers=""):
    """
    交易法门-工具-席位分析-持仓成本
    https://www.jiaoyifamen.com/tools/position/seat
    :param seat: broker name, e.g., seat="永安期货"
    :type seat: str
    :param symbol: e.g., RB
    :type symbol: str
    :param instrument: e.g., rb2005
    :type instrument: str
    :param indicator: e.g., "建仓过程", "净持仓量", "盈亏图"
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定交易日指定机构的持仓结构
    :rtype: pandas.DataFrame
    """
    params = {
        "seat": seat,
        "type": symbol,
        "instrument": instrument,
        "_": int(time.time() * 1000),
    }
    url = "https://www.jiaoyifamen.com/tools/position/interest-process"
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    if indicator == "建仓过程":
        return pd.DataFrame(data_json["kLine"], index=data_json["category"], columns=["open", "close", "low", "high"])
    if indicator == "净持仓量":
        return pd.DataFrame(data_json["neatPosition"], index=data_json["category"], columns=["净持仓量"])
    if indicator == "盈亏图":
        return pd.DataFrame(data_json["profit"], index=data_json["category"], columns=["持仓盈亏"])


# 交易法门-工具-仓单分析
def jyfm_tools_warehouse_receipt_daily(trade_date="2020-01-02", headers=""):
    """
    交易法门-工具-仓单分析-仓单日报
    https://www.jiaoyifamen.com/tools/warehouse-receipt/daily
    :param trade_date: 指定交易日
    :type trade_date: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定交易日的仓单日报数据
    :rtype: pandas.DataFrame
    """
    params = {
        "day": trade_date,
        "_": "1578555328412",
    }
    url = "https://www.jiaoyifamen.com/tools/warehouse-receipt/daily"
    res = requests.get(url, params=params, headers=headers)
    return pd.DataFrame(res.json()["data"])


def jyfm_tools_warehouse_receipt_query(symbol="AL", indicator="仓单数据走势图", headers=""):
    """
    交易法门-工具-仓单分析-仓单查询
    https://www.jiaoyifamen.com/tools/warehouse-receipt/query
    :param symbol: 指定品种
    :type symbol: str
    :param indicator: 指定需要获取的指标, ["仓单数据走势图", "仓单数据季节图"]
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定品种的仓单数量走势图/季节图数据
    :rtype: pandas.DataFrame
    """
    params = {
        "type": symbol,
    }
    url = "https://www.jiaoyifamen.com/tools/warehouse-receipt/query"
    res = requests.get(url, params=params, headers=headers)
    data_json = res.json()
    if indicator == "仓单数据走势图":
        return pd.DataFrame(
            [data_json["category"], data_json["value"], data_json["value2"]]
        ).T
    return pd.DataFrame(
        [
            data_json["dataCategory"],
            data_json["year2013"],
            data_json["year2014"],
            data_json["year2015"],
            data_json["year2016"],
            data_json["year2017"],
            data_json["year2018"],
            data_json["year2019"],
            data_json["year2020"],
        ],
        index=[
            "date",
            "year2013",
            "year2014",
            "year2015",
            "year2016",
            "year2017",
            "year2018",
            "year2019",
            "year2020",
        ],
    ).T


def jyfm_tools_warehouse_virtual_fact_daily(trade_date="2020-01-20", headers=""):
    """
    交易法门-工具-仓单分析-虚实盘比日报
    https://www.jiaoyifamen.com/tools/warehouse-receipt/virtualfact/daily?day=&_=1579532255369
    :param trade_date: 指定日期
    :type trade_date: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定品种指定合约的虚实盘比数据
    :rtype: pandas.DataFrame
    """
    params = {
        "day": trade_date,
        "_": "1579532255370",
    }
    url = "https://www.jiaoyifamen.com/tools/warehouse-receipt/virtualfact/daily"
    res = requests.get(url, params=params, headers=headers)
    data_json = res.json()["data"]
    return pd.DataFrame(data_json)


def jyfm_tools_warehouse_virtual_fact_ratio(symbol="AL", code="05", headers=""):
    """
    交易法门-工具-仓单分析-虚实盘比查询
    https://www.jiaoyifamen.com/tools/warehouse-receipt/ratio
    :param symbol: 指定品种
    :type symbol: str
    :param code: 指定日期的合约
    :type code: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定品种指定合约的虚实盘比数据
    :rtype: pandas.DataFrame
    """
    params = {
        "type": symbol,
        "code": code,
    }
    url = "https://www.jiaoyifamen.com/tools/warehouse-receipt/ratio"
    res = requests.get(url, params=params, headers=headers)
    data_json = res.json()
    return pd.DataFrame(
        [
            data_json["dataCategory"],
            data_json["year2013"],
            data_json["year2014"],
            data_json["year2015"],
            data_json["year2016"],
            data_json["year2017"],
            data_json["year2018"],
            data_json["year2019"],
            data_json["year2020"],
        ],
        index=[
            "date",
            "year2013",
            "year2014",
            "year2015",
            "year2016",
            "year2017",
            "year2018",
            "year2019",
            "year2020",
        ],
    ).T


# 交易法门-工具-期限分析-基差日报
def jyfm_tools_futures_basis_daily(
    trade_date="2020-02-05", indicator="基差率", headers=""
):
    """
    交易法门-工具-期限分析-基差日报
    :param trade_date: 指定交易日期, 注意格式为 "2020-01-02"
    :type trade_date: str
    :param indicator: ["基差率", "基差日报"] 二选一
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定日期的基差日报数据
    :rtype: pandas.DataFrame
    """
    params = {
        "day": trade_date,
    }
    url = "https://www.jiaoyifamen.com/tools/future/basis/daily"
    res = requests.get(url, params=params, headers=headers)
    json_data = res.json()

    if indicator == "基差率":
        # x 轴, y 轴
        return pd.DataFrame(
            [json_data["category"], json_data["value"]], index=["x", "y"]
        ).T

    if indicator == "基差日报":
        return pd.DataFrame(json_data["table_data"])


# 交易法门-工具-期限分析-基差日报-地区选取
def jyfm_tools_futures_basis_daily_area(symbol="Y", headers=""):
    """
    交易法门-工具-期限分析-基差日报-地区选取
    :param symbol: 品种代码
    :type symbol: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定品种的可选地区
    :rtype: list
    """
    params = {
        "type": symbol,
    }
    url = "https://www.jiaoyifamen.com/tools/future/area"
    res = requests.get(url, params=params, headers=headers)
    return res.json()["areas"]


def jyfm_tools_futures_basis_analysis(
    symbol="RB", area="上海", indicator="基差率分布图", headers=""
):
    """
    交易法门-工具-期限分析-基差分析
    :param symbol: 品种代码
    :type symbol: str
    :param area: one of ["上海", "天津"], 不同品种不同日期通过 jyfm_tools_futures_basis_daily_area 返回
    :type area: str
    :param indicator: one of ["基差走势图", "基差率季节图", "基差率分布图"]
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定品种和地区的基差日报
    :rtype: pandas.DataFrame
    """
    url = "https://www.jiaoyifamen.com/tools/future/basis/analysis"
    params = {
        "type": symbol,
        "area": area,
    }
    res = requests.get(url, params=params, headers=headers)
    json_data = res.json()
    if indicator == "基差走势图":
        # x 轴 y 轴
        return pd.DataFrame(
            [json_data["cashValue"], json_data["futureValue"], json_data["basisValue"]],
            columns=json_data["category"],
            index=["现货", "期货", "基差"],
        ).T

    if indicator == "基差率季节图":
        # x 轴 y 轴
        return pd.DataFrame(
            [
                json_data["year2013"],
                json_data["year2014"],
                json_data["year2015"],
                json_data["year2016"],
                json_data["year2017"],
                json_data["year2018"],
                json_data["year2019"],
                json_data["year2020"],
            ],
            index=[
                "year2013",
                "year2014",
                "year2015",
                "year2016",
                "year2017",
                "year2018",
                "year2019",
                "year2020",
            ],
            columns=json_data["dataCategory"],
        ).T

    if indicator == "基差率分布图":
        # x 轴 y 轴
        return pd.DataFrame(
            [json_data["limitCategory"], json_data["limitValue"]], index=["x", "y"]
        ).T


def jyfm_tools_futures_basis_structure(symbol="RB", headers=""):
    """
    交易法门-工具-期限分析-期限结构
    :param symbol: 合约品种
    :type symbol: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 指定品种的期限结构数据
    :rtype: pandas.DataFrame
    """
    params = {
        "type": symbol,
    }
    url = "https://www.jiaoyifamen.com/tools/future/basis/structure"
    res = requests.get(url, params=params, headers=headers)
    return pd.DataFrame(res.json())


def jyfm_tools_futures_basis_rule(
    symbol="RB", code="05", indicator="期货涨跌统计", headers=""
):
    """
    交易法门-工具-期限分析-价格季节性
    :param symbol: 品种
    :type symbol: str
    :param code: 合约具体月份
    :type code: str
    :param indicator: ["期货涨跌统计", "季节性走势图"], 默认为: 期货涨跌统计
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: 期货涨跌统计 or 季节性走势图
    :rtype: pandas.DataFrame
    """
    params = {
        "type": symbol,
        "code": code,
    }
    url = "https://www.jiaoyifamen.com/tools/future/basis/rule"
    res = requests.get(url, params=params, headers=headers)
    data_json = res.json()
    if indicator == "期货涨跌统计":
        return pd.DataFrame(data_json["ratioData"])
    if indicator == "季节性走势图":
        return pd.DataFrame(
            [
                data_json["dataCategory"],
                data_json["year2013"],
                data_json["year2014"],
                data_json["year2015"],
                data_json["year2016"],
                data_json["year2017"],
                data_json["year2018"],
                data_json["year2019"],
                data_json["year2020"],
            ],
            index=[
                "date",
                "2013",
                "2014",
                "2015",
                "2016",
                "2017",
                "2018",
                "2019",
                "2020",
            ],
        ).T


# 交易法门-工具-行情分析-行情数据
def jyfm_tools_futures_market(symbol="RB", code="10", start_date="2020-01-02", end_date="2020-04-02", option="daily", headers=""):
    """
    交易法门-工具-交易规则-限仓规定
    :param symbol: e.g., "RB"
    :type symbol: str
    :param code: e.g., "10"
    :type code: str
    :param start_date: e.g., "2020-01-02"
    :type start_date: str
    :param end_date: e.g., "2020-04-02"
    :type end_date: str
    :param option: e.g., "daily", "weekly", "monthly"
    :type option: str
    :param headers: headers with cookies
    :type headers: dict
    :return:
    :rtype: pandas.DataFrame
    """
    params = {
        "type": symbol,
        "code": code,
        "beginDay": start_date,
        "endDay": end_date,
        "option": option,
        "_": "1585748470772",
    }
    url = "https://www.jiaoyifamen.com/tools/future/market"
    res = requests.get(url, params=params, headers=headers)
    return pd.DataFrame(res.json()["tableData"])


# 交易法门-工具-交易规则
def jyfm_tools_position_limit_info(exchange="CFFEX", headers=""):
    """
    交易法门-工具-交易规则-限仓规定
    :param exchange: one of ["INE", "DCE", "CZCE", "SHFE", "CFFEX"], default is "CFFEX"]
    :type exchange: str
    :param headers: headers with cookies
    :type headers: dict
    :return:
    :rtype: pandas.DataFrame
    """
    params = {
        "page": "1",
        "limit": "10",
        "exchange": exchange,
    }
    url = "https://www.jiaoyifamen.com/tools/position-limit/query"
    res = requests.get(url, params=params, headers=headers)
    return pd.DataFrame(res.json()["data"])


def jyfm_tools_receipt_expire_info(headers=""):
    """
    交易法门-工具-交易规则-仓单有效期
    :param headers: headers with cookies
    :type headers: dict
    :return: all history data
    :rtype: pandas.DataFrame
    """
    temp_df = pd.DataFrame()
    for page in range(1, 4):
        params = {
            "page": str(page),
            "limit": "20",
        }
        res = requests.get(
            f"https://www.jiaoyifamen.com/tools/receipt-expire-info/all",
            params=params,
            headers=headers,
        )
        temp_df = temp_df.append(pd.DataFrame(res.json()["data"]), ignore_index=True)
    return temp_df


def jyfm_tools_symbol_handbook(headers=""):
    """
    交易法门-工具-交易规则-品种手册
    :param headers: headers with cookies
    :type headers: dict
    :return: all history data
    :rtype: pandas.DataFrame
    """
    res = requests.get(
        "https://www.jiaoyifamen.com/tools/receipt-expire-info/variety", headers=headers
    )
    temp_df = pd.DataFrame(res.json()["data"])
    return temp_df


if __name__ == "__main__":
    # 如果要测试函数, 请先在交易法门网站: https://www.jiaoyifamen.com/ 注册帐号密码, 在下面输入对应的帐号和密码后再运行 jyfm_login 函数!
    headers = jyfm_login(account="", password="")

    # 交易法门-工具-套利分析
    jyfm_tools_futures_spread_df = jyfm_tools_futures_spread(
        type_1="RB", type_2="RB", code_1="01", code_2="05", headers=headers, plot=True
    )
    print(jyfm_tools_futures_spread_df)
    jyfm_tools_futures_ratio_df = jyfm_tools_futures_ratio(
        type_1="RB", type_2="RB", code_1="01", code_2="05", headers=headers, plot=True
    )
    print(jyfm_tools_futures_ratio_df)
    jyfm_tools_futures_customize_df = jyfm_tools_futures_customize(
        formula="RB01-1.6*I01-0.5*J01-1200", headers=headers, plot=True
    )
    print(jyfm_tools_futures_customize_df)
    jyfm_tools_futures_full_carry_df = jyfm_tools_futures_full_carry(
        begin_code="05", end_code="09", ratio="4", headers=headers
    )
    print(jyfm_tools_futures_full_carry_df)
    jyfm_tools_futures_arbitrage_matrix_df = jyfm_tools_futures_arbitrage_matrix(
        category="1", type1="RB", type2="RB", headers=headers
    )
    print(jyfm_tools_futures_arbitrage_matrix_df)

    # 交易法门-工具-资讯汇总
    jyfm_tools_research_query_df = jyfm_tools_research_query(
        limit="100", headers=headers
    )
    print(jyfm_tools_research_query_df)
    jyfm_tools_trade_calendar_df = jyfm_tools_trade_calendar(
        trade_date="2020-01-03", headers=headers
    )
    print(jyfm_tools_trade_calendar_df)

    # 交易法门-工具-持仓分析
    jyfm_tools_position_detail_df = jyfm_tools_position_detail(
        symbol="JM", code="jm2005", trade_date="2020-01-03", headers=headers
    )
    print(jyfm_tools_position_detail_df)
    jyfm_tools_position_seat_df = jyfm_tools_position_seat(
        seat="永安期货", trade_date="2020-01-03", headers=headers
    )
    print(jyfm_tools_position_seat_df)
    jyfm_tools_position_season_df = jyfm_tools_position_season(
        symbol="RB", code="05", headers=headers
    )
    print(jyfm_tools_position_season_df)

    # 交易法门-工具-资金分析
    # 交易法门-工具-资金分析-资金流向
    jyfm_tools_position_fund_direction_df = jyfm_tools_position_fund_direction(
        trade_date="2020-02-24", indicator="期货主力合约资金流向排名", headers=headers
    )
    print(jyfm_tools_position_fund_direction_df)
    # 交易法门-工具-资金分析-沉淀资金
    jyfm_tools_position_fund_down_df = jyfm_tools_position_fund_down(
        trade_date="2020-02-24", indicator="期货主力合约沉淀资金排名", headers=headers
    )
    print(jyfm_tools_position_fund_down_df)
    # 交易法门-工具-资金分析-资金季节性
    jyfm_tools_position_fund_season_df = jyfm_tools_position_fund_season(
        symbol="RB", code="05", headers=headers
    )
    print(jyfm_tools_position_fund_season_df)
    # 交易法门-工具-资金分析-成交排名
    jyfm_tools_position_fund_deal_df = jyfm_tools_position_fund_deal(
        trade_date="2020-02-24", indicator="期货主力合约成交量排名", headers=headers
    )
    print(jyfm_tools_position_fund_deal_df)

    # 交易法门-工具-席位分析
    # 交易法门-工具-席位分析-持仓结构
    jyfm_tools_position_structure_df = jyfm_tools_position_structure(
        trade_date="2020-03-02", seat="永安期货", indicator="持仓变化", headers=headers
    )
    print(jyfm_tools_position_structure_df)
    # 交易法门-工具-席位分析-持仓成本
    jyfm_tools_position_seat_cost_df = jyfm_tools_position_seat_cost(seat="永安期货", symbol="RB", code="10", headers=headers)
    print(jyfm_tools_position_seat_cost_df)
    # 交易法门-工具-席位分析-建仓过程
    jyfm_tools_position_interest_process_df = jyfm_tools_position_interest_process(seat="永安期货", symbol="RB", instrument="rb2005", indicator="建仓过程", headers=headers)
    print(jyfm_tools_position_interest_process_df)

    # 交易法门-工具-仓单分析
    # 交易法门-工具-仓单分析-仓单日报
    jyfm_tools_warehouse_receipt_daily_df = jyfm_tools_warehouse_receipt_daily(
        trade_date="2020-01-02", headers=headers
    )
    print(jyfm_tools_warehouse_receipt_daily_df)
    # 交易法门-工具-仓单分析-仓单查询
    jyfm_tools_warehouse_receipt_query_df = jyfm_tools_warehouse_receipt_query(
        symbol="AL", indicator="仓单数据走势图", headers=headers
    )
    print(jyfm_tools_warehouse_receipt_query_df)
    # 交易法门-工具-仓单分析-虚实盘比日报
    jyfm_tools_warehouse_virtual_fact_daily_df = jyfm_tools_warehouse_virtual_fact_daily(
        trade_date="2020-01-20", headers=headers
    )
    print(jyfm_tools_warehouse_virtual_fact_daily_df)
    # 交易法门-工具-仓单分析-虚实盘比查询
    jyfm_tools_warehouse_receipt_ratio_df = jyfm_tools_warehouse_virtual_fact_ratio(
        symbol="AL", code="05", headers=headers
    )
    print(jyfm_tools_warehouse_receipt_ratio_df)

    # 交易法门-工具-期限分析
    jyfm_tools_futures_basis_daily_df = jyfm_tools_futures_basis_daily(
        trade_date="2020-01-02", indicator="基差率", headers=headers
    )
    print(jyfm_tools_futures_basis_daily_df)
    jyfm_tools_futures_basis_analysis_area_df = jyfm_tools_futures_basis_daily_area(
        symbol="Y", headers=headers
    )
    print(jyfm_tools_futures_basis_analysis_area_df)
    jyfm_tools_futures_basis_analysis_df = jyfm_tools_futures_basis_analysis(
        symbol="RB", area="上海", indicator="基差率分布图", headers=headers
    )
    print(jyfm_tools_futures_basis_analysis_df)
    jyfm_tools_futures_basis_structure_df = jyfm_tools_futures_basis_structure(
        symbol="RB", headers=headers
    )
    print(jyfm_tools_futures_basis_structure_df)
    jyfm_tools_futures_basis_rule_df = jyfm_tools_futures_basis_rule(
        symbol="RB", code="05", indicator="期货涨跌统计", headers=headers
    )
    print(jyfm_tools_futures_basis_rule_df)

    # 交易法门-工具-行情分析
    # 交易法门-工具-行情分析-行情数据
    jyfm_tools_futures_market_df = jyfm_tools_futures_market(symbol="RB", code="10", start_date="2020-01-02", end_date="2020-04-02", option="daily", headers=headers)
    print(jyfm_tools_futures_market_df)

    # 交易法门-工具-交易规则
    # 交易法门-工具-交易规则-限仓规定
    jyfm_tools_receipt_expire_info_df = jyfm_tools_receipt_expire_info(headers=headers)
    print(jyfm_tools_receipt_expire_info_df)
    # 交易法门-工具-交易规则-仓单有效期
    jyfm_tools_position_limit_info_df = jyfm_tools_position_limit_info(
        exchange="CFFEX", headers=headers
    )
    print(jyfm_tools_position_limit_info_df)
    # 交易法门-工具-交易规则-品种手册
    jyfm_tools_symbol_handbook_df = jyfm_tools_symbol_handbook(headers=headers)
    print(jyfm_tools_symbol_handbook_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/6/2 16:33
Desc: 两网及退市
http://quote.eastmoney.com/center/gridlist.html#staq_net_board
"""
import pandas as pd
import requests


def stock_staq_net_stop():
    """
    东方财富网-行情中心-沪深个股-两网及退市
    http://quote.eastmoney.com/center/gridlist.html#staq_net_board
    :return: 两网及退市
    :rtype: pandas.DataFrame
    """
    url = 'http://5.push2.eastmoney.com/api/qt/clist/get'
    params = {
        'pn': '1',
        'pz': '2000',
        'po': '1',
        'np': '1',
        'ut': 'bd1d9ddb04089700cf9c27f6f7426281',
        'fltt': '2',
        'invt': '2',
        'fid': 'f3',
        'fs': 'm:0 s:3',
        'fields': 'f12,f14',
        '_': '1622622663841'
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['data']['diff'])
    temp_df.reset_index(inplace=True)
    temp_df['index'] = range(1, len(temp_df)+1)
    temp_df.columns = ['序号', '代码', '名称']
    return temp_df


if __name__ == '__main__':
    stock_staq_net_stop_df = stock_staq_net_stop()
    print(stock_staq_net_stop_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2021/9/16 17:29
Desc: 巨潮资讯-个股-历史分红
http://webapi.cninfo.com.cn/#/company?companyid=600009
"""
import time
from py_mini_racer import py_mini_racer
import requests
import pandas as pd


js_str = """
    function mcode(input) {  
                var keyStr = "ABCDEFGHIJKLMNOP" + "QRSTUVWXYZabcdef" + "ghijklmnopqrstuv"   + "wxyz0123456789+/" + "=";  
                var output = "";  
                var chr1, chr2, chr3 = "";  
                var enc1, enc2, enc3, enc4 = "";  
                var i = 0;  
                do {  
                    chr1 = input.charCodeAt(i++);  
                    chr2 = input.charCodeAt(i++);  
                    chr3 = input.charCodeAt(i++);  
                    enc1 = chr1 >> 2;  
                    enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);  
                    enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);  
                    enc4 = chr3 & 63;  
                    if (isNaN(chr2)) {  
                        enc3 = enc4 = 64;  
                    } else if (isNaN(chr3)) {  
                        enc4 = 64;  
                    }  
                    output = output + keyStr.charAt(enc1) + keyStr.charAt(enc2)  
                            + keyStr.charAt(enc3) + keyStr.charAt(enc4);  
                    chr1 = chr2 = chr3 = "";  
                    enc1 = enc2 = enc3 = enc4 = "";  
                } while (i < input.length);  
          
                return output;  
            }  
"""


def stock_dividents_cninfo(symbol: str = "600009") -> pd.DataFrame:
    """
    巨潮资讯-个股-历史分红
    http://webapi.cninfo.com.cn/#/company?companyid=600009
    :param symbol: 股票代码
    :type symbol: str
    :return: 历史分红
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1139"
    params = {
        'scode': symbol
              }
    random_time_str = str(int(time.time()))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(js_str)
    mcode = js_code.call("mcode", random_time_str)
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "mcode": mcode,
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.columns = [
        "实施方案公告日期",
        "送股比例",
        "转增比例",
        "派息比例",
        "股权登记日",
        "除权日",
        "派息日",
        "股份到账日",
        "实施方案分红说明",
        "分红类型",
        "报告时间",
    ]
    temp_df["实施方案公告日期"] = pd.to_datetime(temp_df["实施方案公告日期"]).dt.date
    temp_df["送股比例"] = pd.to_numeric(temp_df["送股比例"], errors="coerce")
    temp_df["转增比例"] = pd.to_numeric(temp_df["转增比例"], errors="coerce")
    temp_df["派息比例"] = pd.to_numeric(temp_df["派息比例"], errors="coerce")
    temp_df["股权登记日"] = pd.to_datetime(temp_df["股权登记日"], errors="coerce").dt.date
    temp_df["除权日"] = pd.to_datetime(temp_df["除权日"], errors="coerce").dt.date
    temp_df["派息日"] = pd.to_datetime(temp_df["派息日"], errors="coerce").dt.date
    return temp_df


if __name__ == "__main__":
    stock_dividents_cninfo_df = stock_dividents_cninfo(symbol="600009")
    print(stock_dividents_cninfo_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/1/20 11:48
Desc: 腾讯-债券-质押式回购-实时行情-成交明细
下载成交明细-每个交易日 16:00 提供当日数据
http://stockhtm.finance.qq.com/sstock/ggcx/131802.shtml
"""
from io import StringIO

import pandas as pd
import requests


def bond_repo_zh_tick(
    code: str = "sz131802", trade_date: str = "20201028"
) -> pd.DataFrame:
    """
    成交明细-每个交易日 16:00 提供当日数据
    http://stockhtm.finance.qq.com/sstock/ggcx/131802.shtml
    :param code: 带市场标识的债券-质押式回购代码
    :type code: str
    :param trade_date: 需要提取数据的日期
    :type trade_date: str
    :return: 返回指定交易日债券-质押式回购成交明细的数据
    :rtype: pandas.DataFrame
    """
    url = "http://stock.gtimg.cn/data/index.php"
    params = {
        "appn": "detail",
        "action": "download",
        "c": code,
        "d": trade_date,
    }
    r = requests.get(url, params=params)
    r.encoding = "gbk"
    temp_df = pd.read_table(StringIO(r.text))
    return temp_df


if __name__ == "__main__":
    date_list = pd.date_range(start="20210926", end="20210928").tolist()
    date_list = [item.strftime("%Y%m%d") for item in date_list]
    for item in date_list:
        data = bond_repo_zh_tick(code="sz131802", trade_date=f"{item}")
        if not data.empty:
            print(data)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/4/6 15:19
Desc: 东方财富网-数据中心-新股数据-注册制审核
http://data.eastmoney.com/kcb/?type=nsb
"""
import pandas as pd
import requests


def stock_register_kcb() -> pd.DataFrame:
    """
    东方财富网-数据中心-新股数据-注册制审核-科创板
    http://data.eastmoney.com/kcb/?type=nsb
    :return: 科创板注册制审核结果
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/get"
    params = {
        'st': 'UPDATE_DATE',
        'sr': '-1',
        'ps': '5000',
        'p': '1',
        'type': 'RPT_REGISTERED_INFO',
        'sty': 'ORG_CODE,ORG_CODE_OLD,ISSUER_NAME,CHECK_STATUS,CHECK_STATUS_CODE,REG_ADDRESS,CSRC_INDUSTRY,RECOMMEND_ORG,LAW_FIRM,ACCOUNT_FIRM,UPDATE_DATE,ACCEPT_DATE,TOLIST_MARKET,SECURITY_CODE',
        'token': '894050c76af8597a853f5b408b759f5d',
        'client': 'WEB',
        'filter': '(TOLIST_MARKET="科创板")',
    }
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.114 Safari/537.36'
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    page_num = data_json['result']['pages']
    big_df = pd.DataFrame()
    for page in range(1, page_num+1):
        params = {
            'st': 'UPDATE_DATE',
            'sr': '-1',
            'ps': '5000',
            'p': page,
            'type': 'RPT_REGISTERED_INFO',
            'sty': 'ORG_CODE,ORG_CODE_OLD,ISSUER_NAME,CHECK_STATUS,CHECK_STATUS_CODE,REG_ADDRESS,CSRC_INDUSTRY,RECOMMEND_ORG,LAW_FIRM,ACCOUNT_FIRM,UPDATE_DATE,ACCEPT_DATE,TOLIST_MARKET,SECURITY_CODE',
            'token': '894050c76af8597a853f5b408b759f5d',
            'client': 'WEB',
            'filter': '(TOLIST_MARKET="科创板")',
        }
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.114 Safari/537.36'
        }
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json['result']["data"])
        big_df = big_df.append(temp_df, ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df['index'] = range(1, len(big_df) + 1)
    big_df.columns = [
        "序号",
        "_",
        "_",
        "发行人全称",
        "审核状态",
        "_",
        "注册地",
        "证监会行业",
        "保荐机构",
        "律师事务所",
        "会计师事务所",
        "更新日期",
        "受理日期",
        "拟上市地点",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "发行人全称",
            "审核状态",
            "注册地",
            "证监会行业",
            "保荐机构",
            "律师事务所",
            "会计师事务所",
            "更新日期",
            "受理日期",
            "拟上市地点",
        ]
    ]
    return big_df


def stock_register_cyb() -> pd.DataFrame:
    """
    东方财富网-数据中心-新股数据-注册制审核-创业板
    http://data.eastmoney.com/xg/cyb/
    :return: 创业板注册制审核结果
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/get"
    params = {
        'st': 'UPDATE_DATE',
        'sr': '-1',
        'ps': '5000',
        'p': '1',
        'type': 'RPT_REGISTERED_INFO',
        'sty': 'ORG_CODE,ORG_CODE_OLD,ISSUER_NAME,CHECK_STATUS,CHECK_STATUS_CODE,REG_ADDRESS,CSRC_INDUSTRY,RECOMMEND_ORG,LAW_FIRM,ACCOUNT_FIRM,UPDATE_DATE,ACCEPT_DATE,TOLIST_MARKET,SECURITY_CODE',
        'token': '894050c76af8597a853f5b408b759f5d',
        'client': 'WEB',
        'filter': '(TOLIST_MARKET="创业板")',
    }
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.114 Safari/537.36'
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    page_num = data_json['result']['pages']
    big_df = pd.DataFrame()
    for page in range(1, page_num+1):
        params = {
            'st': 'UPDATE_DATE',
            'sr': '-1',
            'ps': '5000',
            'p': page,
            'type': 'RPT_REGISTERED_INFO',
            'sty': 'ORG_CODE,ORG_CODE_OLD,ISSUER_NAME,CHECK_STATUS,CHECK_STATUS_CODE,REG_ADDRESS,CSRC_INDUSTRY,RECOMMEND_ORG,LAW_FIRM,ACCOUNT_FIRM,UPDATE_DATE,ACCEPT_DATE,TOLIST_MARKET,SECURITY_CODE',
            'token': '894050c76af8597a853f5b408b759f5d',
            'client': 'WEB',
            'filter': '(TOLIST_MARKET="创业板")',
        }
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.114 Safari/537.36'
        }
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json['result']["data"])
        big_df = big_df.append(temp_df, ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df['index'] = range(1, len(big_df) + 1)
    big_df.columns = [
        "序号",
        "_",
        "_",
        "发行人全称",
        "审核状态",
        "_",
        "注册地",
        "证监会行业",
        "保荐机构",
        "律师事务所",
        "会计师事务所",
        "更新日期",
        "受理日期",
        "拟上市地点",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "发行人全称",
            "审核状态",
            "注册地",
            "证监会行业",
            "保荐机构",
            "律师事务所",
            "会计师事务所",
            "更新日期",
            "受理日期",
            "拟上市地点",
        ]
    ]
    return big_df


def stock_register_db() -> pd.DataFrame:
    """
    东方财富网-数据中心-新股数据-注册制审核-达标企业
    http://data.eastmoney.com/xg/cyb/
    :return: 达标企业
    :rtype: pandas.DataFrame
    """
    url = "http://dcfm.eastmoney.com/EM_MutiSvcExpandInterface/api/js/get"
    params = {
        'st': 'eutime',
        'sr': '-1',
        'ps': '5000',
        'p': '1',
        'type': 'KCB_LB',
        'js': '{"data":(x),"pages":(tp)}',
        'token': '894050c76af8597a853f5b408b759f5d',
        'filter': "(MKT='fnsb')",
    }
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.114 Safari/537.36'
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    page_num = data_json['pages']
    big_df = pd.DataFrame()
    for page in range(1, page_num+1):
        params = {
            'st': 'eutime',
            'sr': '-1',
            'ps': '5000',
            'p': page,
            'type': 'KCB_LB',
            'js': '{"data":(x),"pages":(tp)}',
            'token': '894050c76af8597a853f5b408b759f5d',
            'filter': "(MKT='fnsb')",
        }
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.114 Safari/537.36'
        }
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json['data'])
        big_df = big_df.append(temp_df, ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df['index'] = range(1, len(big_df) + 1)
    big_df.columns = [
        "序号",
        "_",
        "_",
        "_",
        "企业名称",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "经营范围",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "近三年营业收入-2019",
        "近三年净利润-2019",
        "近三年研发费用-2019",
        "近三年营业收入-2018",
        "近三年净利润-2018",
        "近三年研发费用-2018",
        "近三年营业收入-2017",
        "近三年净利润-2017",
        "近三年研发费用-2017",
        "近两年累计净利润",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "企业名称",
            "经营范围",
            "近三年营业收入-2019",
            "近三年净利润-2019",
            "近三年研发费用-2019",
            "近三年营业收入-2018",
            "近三年净利润-2018",
            "近三年研发费用-2018",
            "近三年营业收入-2017",
            "近三年净利润-2017",
            "近三年研发费用-2017",
            "近两年累计净利润",
        ]
    ]
    return big_df


if __name__ == "__main__":
    stock_register_kcb_df = stock_register_kcb()
    print(stock_register_kcb_df)

    stock_register_cyb_df = stock_register_cyb()
    print(stock_register_cyb_df)

    stock_register_db_df = stock_register_db()
    print(stock_register_db_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2019/10/25 15:56
Desc: 和讯财经-上市公司社会责任报告数据, http://stockdata.stock.hexun.com/zrbg/
"""
from mssdk.utils import demjson
import requests
import pandas as pd
from tqdm import tqdm

from mssdk.stock.cons import (hx_headers,
                                hx_params,
                                hx_url)


def stock_zh_a_scr_report(report_year=2018, page=1):
    """
    获取和讯财经-上市公司社会责任报告数据, 从2010年至今(年度)
    因为股票数量大, 所以获取某年需要遍历所有页
    :param report_year: int 年份
    :param page: int 具体某页
    :return: pandas.DataFrame
            股票名称   股东责任    总得分 等级  员工责任  环境责任   社会责任 供应商、客户和消费者权益责任
    0    陆家嘴(600663)  23.97  42.97  C  4.00  0.00  15.00           0.00
    1   世荣兆业(002016)  24.61  42.44  C  2.83  0.00  15.00           0.00
    2    万科A(000002)  23.18  42.18  C  4.00  0.00  15.00           0.00
    3   华夏幸福(600340)  22.76  41.76  C  4.00  0.00  15.00           0.00
    4   万业企业(600641)  22.03  41.03  C  4.00  0.00  15.00           0.00
    5   华联控股(000036)  24.96  40.98  C  1.02  0.00  15.00           0.00
    6   中国国贸(600007)  22.43  40.98  C  3.55  0.00  15.00           0.00
    7    新黄浦(600638)  22.25  40.92  C  3.67  0.00  15.00           0.00
    8   金科股份(000656)  22.47  40.62  C  4.00  0.00  14.15           0.00
    9   中华企业(600675)  21.44  40.44  C  4.00  0.00  15.00           0.00
    10  京投发展(600683)  21.39  40.39  C  4.00  0.00  15.00           0.00
    11  浦东金桥(600639)  21.32  40.32  C  4.00  0.00  15.00           0.00
    12  中航善达(000043)  22.80  40.06  C  2.26  0.00  15.00           0.00
    13   新华联(000620)  21.00  40.00  D  4.00  0.00  15.00           0.00
    14  首开股份(600376)  20.77  39.77  D  4.00  0.00  15.00           0.00
    15  深物业A(000011)  23.49  39.73  D  1.50  0.00  14.74           0.00
    16  江苏租赁(600901)  20.62  39.62  D  4.00  0.00  15.00           0.00
    17  中国太保(601601)  20.61  39.61  D  4.00  0.00  15.00           0.00
    18  中国平安(601318)  20.59  39.59  D  4.00  0.00  15.00           0.00
    19  深深房A(000029)  22.45  39.47  D  2.02  0.00  15.00           0.00
    """
    hx_params_copy = hx_params.copy()
    hx_params_copy.update({"date": "{}-12-31".format(str(report_year))})
    hx_params_copy.update({"page": page})
    res = requests.get(hx_url, headers=hx_headers, params=hx_params_copy)
    temp_df = res.text[res.text.find("(") + 1:res.text.rfind(")")]
    py_obj = demjson.decode(temp_df)
    industry = [item["industry"] for item in py_obj["list"]]
    stock_number = [item["stockNumber"] for item in py_obj["list"]]
    industry_rate = [item["industryrate"] for item in py_obj["list"]]
    price_limit = [item["Pricelimit"] for item in py_obj["list"]]
    looting_chips = [item["lootingchips"] for item in py_obj["list"]]
    r_scramble = [item["rscramble"] for item in py_obj["list"]]
    strong_stock = [item["Strongstock"] for item in py_obj["list"]]
    s_cramble = [item["Scramble"] for item in py_obj["list"]]
    return pd.DataFrame([industry, stock_number, industry_rate, price_limit, looting_chips, r_scramble, strong_stock, s_cramble],
                        index=["股票名称", "股东责任", "总得分", "等级", "员工责任", "环境责任", "社会责任", "供应商、客户和消费者权益责任"]).T


if __name__ == "__main__":
    for i_page in tqdm(range(1, 100)):
        df = stock_zh_a_scr_report(report_year=2018, page=i_page)
        # print(df)



# coding=utf-8
#!/usr/bin/env python
"""
Date: 2020/4/19 14:27
Desc: 西本新干线-指数数据
http://nanjing.96369.net/
"""
import random
from io import BytesIO

import matplotlib.pyplot as plt
import pandas as pd
import requests
from PIL import Image
from bs4 import BeautifulSoup

from mssdk.futures_derivative.cons import (
    xgx_short_headers,
    xgx_headers,
    xgx_code_url,
    xgx_main_url,
    symbol_dict,
)

plt.rcParams["font.sans-serif"] = "SimHei"
plt.rcParams["axes.unicode_minus"] = False


def _get_code_pic():
    payload = {"": round(random.random(), 16)}
    session = requests.session()
    res = session.get(xgx_code_url, params=payload, headers=xgx_short_headers)
    f = Image.open(BytesIO(res.content))
    f.show()
    return session


def futures_xgx_index(
    symbol: str = 67,
    start_date: str = "2000-10-01",
    end_date: str = "2020-04-17",
    plot: bool = True,
) -> pd.DataFrame:
    session = _get_code_pic()
    value = input()
    payload = {"txtStartTime": start_date, "txtEndTime": end_date, "txtyzcode": value}
    res = session.post(xgx_main_url.format(symbol), data=payload, headers=xgx_headers)
    soup = BeautifulSoup(res.text, "lxml")

    table_df = pd.read_html(res.text)[0]
    table_df.index = pd.to_datetime(table_df["日期"])
    del table_df["日期"]
    if plot:
        table_df["值"].plot()
        plt.title(
            soup.find("div", attrs={"class": "commodity_right"}).find("h5").get_text()
        )
        plt.xlabel("日期")
        plt.ylabel("值")
        plt.show()
        return table_df
    else:
        return table_df


if __name__ == "__main__":
    # 国内螺纹钢社会库存量 67
    # 国内线材社会库存量 68
    symbol_dict_df = pd.DataFrame.from_dict(symbol_dict, orient="index")
    print(symbol_dict_df)
    futures_xgx_index_df = futures_xgx_index(
        symbol=161, start_date="2000-10-01", end_date="2020-04-17", plot=True
    )
    print(futures_xgx_index_df)
